/*
 2 nested calls (1+ function itself) on all low-level routines +
 	HT1632_setPWM()
 	HT1632_fill()
 	HT1632_cls()
 	HT1632_string()
 3 nested calls on all remaining high-level routines 
*/
#include "derivative.h"
#include "pin_mapping.h"

#define CS_LO IO_DIO8_DTR=0
#define CS_HI	IO_DIO8_DTR=1
#define RD_LO	IO_DIO12_CD=0
#define RD_HI	IO_DIO12_CD=1
#define WR_LO	IO_DIO6_RTS_HOST=0
#define WR_HI	IO_DIO6_RTS_HOST=1
#define DATA  IO_DIO7_CTS_HOST
#define DATA_LO IO_DIO7_CTS_HOST=0
#define DATA_HI IO_DIO7_CTS_HOST=1
#define DATA_ISOUT IO_DIO7_CTS_HOST_D=1
#define DATA_ISIN	 IO_DIO7_CTS_HOST_D=0
#define CTRLC IO_DIO7_CTS_HOST_PE=1;IO_DIO8_DTR_D=1;IO_DIO12_CD_D=1;IO_DIO6_RTS_HOST_D=1;CS_HI;RD_HI;WR_HI    // control lines high

//#define SWAP(a) _swap(&a)
//#define SWAP(a) a=(a<<4)|((uint8_t)a>>4)
#define SWAP(a) __asm LDA a ;__asm NSA ;__asm STA a 

typedef unsigned char uint8_t;
typedef unsigned int uint16_t;

#include "HT1632.h"

// INTERNAL ROUTINES
void send_cbits(uint8_t cmdbit,uint8_t bits)
{
	DATA_ISOUT;
	while(bits--){
		WR_LO;
		if(cmdbit&0x80)
			DATA_HI;
		else
			DATA_LO;
		WR_HI;
		cmdbit<<=1;
	}	
	DATA_ISIN;
}

uint8_t recv_4dbits(void)
{
uint8_t datbit=0,bits=4;

	while(bits--){
		RD_LO;
		if(DATA)
			datbit|=(1<<3);
		else
			datbit&=~(1<<3);
		RD_HI;
		datbit>>=1;
	}
	return(datbit);
}

void send_4dbits(uint8_t datbit)
{
uint8_t bits=4;

	DATA_ISOUT;
	while(bits--){
		WR_LO;
		if(datbit&0x01)
			DATA_HI;
		else
			DATA_LO;
		WR_HI;
		datbit>>=1;
	}	
	DATA_ISIN;
}

// LOW-LEVEL ROUTINES
uint8_t HT1632_read(uint8_t address)
{
uint8_t data;

	CS_LO;
	send_cbits(HT1632_READ,3);
	send_cbits(address<<1,7);
	data=recv_4dbits();
	CS_HI;
	return(data);
}

void HT1632_write(uint8_t address,uint8_t data)
{
	CS_LO;
	send_cbits(HT1632_WRITE,3);
	send_cbits(address<<1,7);
	send_4dbits(data);
	CS_HI;
}

void HT1632_command(uint8_t cmd)
{
	CS_LO;
	send_cbits(HT1632_CMD,3);
	send_cbits(cmd,9);
	CS_HI;
}

void HT1632_readbytes(uint8_t *bytes,uint8_t num)
{
register uint8_t aux=(uint8_t)bytes;

	CS_LO;
	send_cbits(HT1632_READ,3);
	send_cbits(aux<<1,7);
	while(num--)
		*(bytes++)=recv_4dbits();
	CS_HI;
}

void HT1632_writebytes(uint8_t *bytes,uint8_t num)
{
register uint8_t aux=(uint8_t)bytes;

	CS_LO;
	send_cbits(HT1632_WRITE,3);
	send_cbits(aux<<1,7);
	while(num--)
		send_4dbits(*(bytes++));
	CS_HI;
}

// HIGH-LEVEL ROUTINES
void init_HT1632(void)
{
	CTRLC;				// set ins and outs, put control lines high
	DATA_ISIN;
	HT1632_command(HT1632_CMD_SYS_DIS);
	HT1632_command(HT1632_CMD_COM_OPT_N8);
	HT1632_command(HT1632_CMD_MASTER_MODE);
	HT1632_command(HT1632_CMD_CLK_RC);
	HT1632_command(HT1632_CMD_SYS_EN);
	HT1632_setPWM(15);
	HT1632_command(HT1632_CMD_LED_ON);
}

void HT1632_fill(uint8_t pattern)
{
uint8_t i=64;

	CS_LO;
	send_cbits(HT1632_WRITE,3);
	send_cbits(0,7);
	send_4dbits(pattern);
	while(--i){
		send_4dbits(pattern);
	}	
	CS_HI;
}

#include "font5x7.h"
void HT1632_string(uint8_t address,char *string)
{
char c;
uint16_t addr;
uint8_t cnt,aux;

	CS_LO;
	send_cbits(HT1632_WRITE,3);
	send_cbits(address<<1,7);
#pragma MESSAGE DISABLE C5909   // so CW stops complaining on next statement
	while(c=*(string++)){
		addr=(uint16_t)5*(c-' ');
		cnt=5;
		for(cnt=0;cnt<5;cnt++){
			aux=FONT5x7[addr+cnt];
			aux>>=1;		// char set is moved down and this hw has only 7 LEDs
			send_4dbits(aux);
			SWAP(aux);
			send_4dbits(aux);
		}
	}
	CS_HI;
}
